-- HugeBagWindow.lua
-- Originally TheOneBagWindow.lua written by Rod, extended by Habna

-- HugeBag Knowed bugs:
-- when right clicking on tools the icon is not swaping properly

-- HugeBog future functions:
--

import "Turbine";
import "Turbine.Gameplay";
import "Turbine.UI";
import "Turbine.UI.Extensions";
import "Turbine.UI.Lotro";
import "Turbine.Utils";

frmMain = class( Turbine.UI.Extensions.Window );

freeslots = 0; --> ** Global Variable to get the number of free slots **
backpackSize = self; --> ** Global variable to get the number of items in all 5 bags **
InitXPos = 17; --> ** Initial X position of the first Item in the window **
InitYPos = 33; --> ** Initial Y position of the first Item in the window **
width, height = self;
edgePadding = 15;
Version = self;
StartVisible = true;
ShowSlotsInfo = true;
WinLocX, WinLocY = self;
WinWidth, WinHeight = self;
settings = self;
--Turbine.Shell.WriteLine( WinWidth );

-- **v Constructor v**
function frmMain:Constructor()
	Turbine.UI.Extensions.Window.Constructor( self );
	
	self:LoadSetting();
	self:SetVisible( StartVisible );

	-- **v Setting to position the window at the bottom right of the screen v**
	self:SetSize( WinWidth, WinHeight );
	self:SetBackColor( Turbine.UI.Color() );
	self:SetPosition( WinLocX, WinLocY );
	self:SetOpacity( 1.0 );
	self:SetAllowDrop( true );
	self:SetFadeSpeed( 0.4 );
	-- **^
	-- **v Set the opacity when the mouse enter the window v**
	self.MouseEnter = function( sender, args )
		sender:SetOpacity( 1.0 );
	end
	-- **^
	-- **v Set the opacity when the mouse leave the window v**
	self.MouseLeave = function( sender, args )
		sender:SetOpacity( 1.0 );
	end
	-- **^

	self:SetWantsKeyEvents( true );
	
	-- **v Detect key press v**
	self.KeyDown = function( sender, args )
		-- **v Hide windows if ESC key is press v**
		if ( args.Action == Turbine.UI.Lotro.Action.Escape ) then
			sender:SetVisible( true )
		end
		-- **^
		-- **v Show/Hide window if any bag was toggle v**
		if ( args.Action == Turbine.UI.Lotro.Action.ToggleBags or
		     args.Action == Turbine.UI.Lotro.Action.ToggleBag1 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag2 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag3 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag4 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag5 )
		then
			sender:SetVisible( not sender:IsVisible() ) 
		end
		-- **^
	end
	-- **^
	-- **v Set the scrollbar v**
	self.itemListBoxScrollBar = Turbine.UI.Lotro.ScrollBar();
	self.itemListBoxScrollBar:SetOrientation( Turbine.UI.Orientation.Vertical );
	self.itemListBoxScrollBar:SetParent( self );
	-- **^
	-- **v Set the resizehandle v**
	self.resizeHandle = Turbine.UI.Control();
	self.resizeHandle:SetParent( self );
	self.resizeHandle:SetZOrder( 100 );
	self.resizeHandle:SetSize( 20, 20 );
	self.resizeHandle:SetPosition( self:GetWidth() - self.resizeHandle:GetWidth(), self:GetHeight() - self.resizeHandle:GetHeight() );

	self.resizeHandle.MouseDown = function( sender, args )
		sender.dragStartX = args.X;
		sender.dragStartY = args.Y;
		sender.dragging = true;
	end

	self.resizeHandle.MouseUp = function( sender, args )
		sender.dragging = false;
		frmMain:SaveSetting()
	end

	self.resizeHandle.MouseMove = function( sender, args )
		width, height = self:GetSize();

		if ( sender.dragging ) then
			self:SetSize( width + ( args.X - sender.dragStartX ), height + ( args.Y - sender.dragStartY ) );
			sender:SetPosition( self:GetWidth() - sender:GetWidth(), self:GetHeight() - sender:GetHeight() )
			self:PerformLayout()
		end
	end
	-- **^
	-- **v Set the listbox v**
	self.itemListBox = Turbine.UI.ListBox();
	self.itemListBox:SetParent( self );
	self.itemListBox:SetOrientation( Turbine.UI.Orientation.Horizontal );
	self.itemListBox:SetVerticalScrollBar( self.itemListBoxScrollBar );
	-- **^	
			
	self.items = { }; --> ** Initialize the container that holds the elements referencing the player's inventory **

	-- **v Get player gameplay v**
	if pcall( Turbine.Gameplay.LocalPlayer.GetInstance ) then
		player = Turbine.Gameplay.LocalPlayer.GetInstance();
	else
		player = Turbine.Gameplay.LocalPlayer();
	end
	-- **^

	self.backpack = player:GetBackpack(); --> ** Initialize the backpack with the current player gameplay **

	self:Refresh(); --> ** Refresh the backpack list **

	-- **v Backpack size has changed v**
	self.backpack.SizeChanged = function( sender, args )
		self:Refresh();
	end
	-- **^
	-- **v Adding an item v**
	self.backpack.ItemAdded = function( sender, args )
		self.items[args.Index]:SetItem( self.backpack:GetItem( args.Index ) );
		freeslots = freeslots -1;
		self:PrintTitle()
	end
	-- **^
	-- **v Removing an item v**
	self.backpack.ItemRemoved = function( sender, args )
		self.items[args.Index]:SetItem( self.backpack:GetItem( args.Index ) );
		freeslots = freeslots +1;
		self:PrintTitle()
	end
	-- **^
	-- **v Changing place an item v**
	self.backpack.ItemMoved = function( sender, args )
		self.items[args.OldIndex]:SetItem( self.backpack:GetItem( args.OldIndex ) );
		self.items[args.NewIndex]:SetItem( self.backpack:GetItem( args.NewIndex ) );
	end

	self.DragDrop = function( sender, args )
		local shortcut = args.DragDropInfo:GetShortcut();

		if ( shortcut ~= nil ) then
			local destinationItemControl = self.itemListBox:GetItemAt( args.X - InitXPos, args.Y - InitYPos );

			-- if nil, then it's out of range, so don't do anything
			if ( destinationItemControl ~= nil ) then
				local destinationIndex = self.itemListBox:IndexOfItem( destinationItemControl );
				self.backpack:PerformShortcutDrop(shortcut, destinationIndex, Turbine.UI.Control.IsShiftKeyDown() );
			end
		end
	end
	-- **^
end
-- **^
-- **v Refresh the X,Y positiov when window position change v**
function frmMain:PositionChanged(args)
	local left, top = self:GetPosition()

	WinLocX = left
	WinLocY = top

	frmMain:SaveSetting()
end
-- **^
-- **v Refresh the backpack list v**
function frmMain:Refresh()
	backpackSize = self.backpack:GetSize();
	local temp;

	for i = 1, backpackSize, 1 do
		if ( self.items[i] ) then
			self.items[i]:SetParent( nil );
		end

		-- Get item and set background color
		temp = Turbine.UI.Lotro.ItemControl( self.backpack:GetItem( i ) );
		temp:SetBackground( "HabnaPlugins/HugeBag/Resources/slotBackground.tga" );
		temp:SetBlendMode( Turbine.UI.BlendMode.Overlay );
		
		self.items[i] = temp;
		--self.items[i] = Turbine.UI.Lotro.ItemControl( self.backpack:GetItem( i ) );
		self.itemListBox:AddItem( self.items[i] );
	end
	self:PerformLayout();
end
-- **^
-- **v Layout of the items in the listbox v**
function frmMain:PerformLayout()
	width, height = self:GetSize();
	WinLocX, WinLocY = self:GetPosition()
	local itemWidth = 40;
	
	if ( self.items[1] ~= nil ) then
		itemWidth = self.items[1]:GetWidth()
	end

	local listWidth = width - 40;
	local listHeight = height - 50;
	local itemsPerRow = listWidth / itemWidth;
	
	self.itemListBox:SetPosition( InitXPos, InitYPos );
	self.itemListBox:SetSize( listWidth, listHeight );
	self.itemListBox:SetMaxItemsPerLine( itemsPerRow );
	
	self.itemListBoxScrollBar:SetPosition( width - 17, 40 );
	self.itemListBoxScrollBar:SetSize( 10, listHeight - 15 );

	self:CountFreeSlots()
end
-- **^
-- **v Do stuff before closing the windows v**
function frmMain:Closing(args)
	self:SaveSetting();
end
-- **^
-- **v Count the free slots v**
function frmMain:CountFreeSlots()
	freeslots = 0;

	for i = 1, backpackSize, 1 do
		if ( self.backpack:GetItem( i ) == nil ) then
			freeslots = freeslots + 1;
		end
	end

	self:PrintTitle()
end
-- **^
-- **v Print the title with the free slots and the max v**
function frmMain:PrintTitle()
	if ShowSlotsInfo == true then
		self:SetText( "HugeBag - free slots: " .. freeslots .. " / " .. backpackSize );
	else
		self:SetText( "HugeBag" );
	end
end
-- **^
-- **v Load settings v**
function frmMain:LoadSetting()
	settings = {};
	settings = Turbine.PluginData.Load( Turbine.DataScope.Character, "HugeBagSettings" );
	
	if settings == nil then
		WinWidth = 460;
		WinHeight = 300;
		self:SetSize( WinWidth, WinHeight );
		WinLocX = Turbine.UI.Display.GetWidth() - edgePadding - WinWidth
		WinLocY = Turbine.UI.Display.GetHeight() - edgePadding - WinHeight - 55
		self:SaveSetting()
	end

	Version = settings.Version;
	StartVisible = settings.StartVisible;
	ShowSlotsInfo = settings.ShowSlotsInfo;
	WinLocX = settings.X;
	WinLocY = settings.Y;
	WinWidth = settings.Width;
	WinHeight = settings.Height;
end
-- **^
-- **v Save settings v**
function frmMain:SaveSetting()
	settings = {};
	settings.Version = "1.0.0"; -- current version
	settings.StartVisible = StartVisible;
	settings.ShowSlotsInfo = ShowSlotsInfo;
	settings.X = WinLocX; --Turbine.UI.Display.GetWidth() - edgePadding - self:GetWidth()
	settings.Y = WinLocY; --Turbine.UI.Display.GetHeight() - edgePadding - self:GetHeight() - 55
	settings.Width = self:GetWidth();
	settings.Height = self:GetHeight();

	Turbine.PluginData.Save( Turbine.DataScope.Character, "HugeBagSettings", settings );
end
-- **^